@@ -1,11 +1,16 @@
+import socket
+from threading import Thread
+from time import sleep
 import zmq
+from psutil import pids
 import json
 import os
 from subprocess import Popen
 from typing import List, Union, Optional, Tuple, Dict
 from tdw.librarian import ModelLibrarian, SceneLibrarian, MaterialLibrarian, HDRISkyboxLibrarian, \
     HumanoidAnimationLibrarian, HumanoidLibrarian, HumanoidAnimationRecord, RobotLibrarian
-from tdw.output_data import OutputData, Version
+from tdw.backend.paths import EDITOR_LOG_PATH, PLAYER_LOG_PATH
+from tdw.output_data import OutputData, Version, QuitSignal
 from tdw.release.build import Build
 from tdw.release.pypi import PyPi
 from tdw.version import __version__
@@ -24,13 +29,14 @@ class Controller(object):
     ```
     """
 
-    def __init__(self, port: int = 1071, check_version: bool = True, launch_build: bool = True):
+    def __init__(self, port: int = 1071, check_version: bool = True, launch_build: bool = True, udp: bool = True):
         """
         Create the network socket and bind the socket to the port.
 
         :param port: The port number.
         :param check_version: If true, the controller will check the version of the build and print the result.
         :param launch_build: If True, automatically launch the build. If one doesn't exist, download and extract the correct version. Set this to False to use your own build, or (if you are a backend developer) to use Unity Editor.
+        :param udp: If True, start a UDP heartbeat with the build. If the heartbeat stops, the controller will quit.
         """
 
         # Compare the installed version of the tdw Python module to the latest on PyPi.
@@ -40,9 +46,7 @@ class Controller(object):
 
         # Launch the build.
         if launch_build:
-            Controller.launch_build(port = port)
-
-
+            Controller.launch_build(port=port)
         context = zmq.Context()
 
         self.socket = context.socket(zmq.REP)
@@ -50,6 +54,43 @@ class Controller(object):
 
         self.socket.recv()
 
+        udp_socket: Optional[socket.socket] = None
+
+        # Set error handling to default values (the build will try to quit on errors and exceptions).
+        # Request the version to log it and remember here if the Editor is being used.
+        start_commands = [{"$type": "set_error_handling"},
+                          {"$type": "send_version"}]
+        # Start the UDP heartbeat.
+        if udp:
+            # Start a UDP heartbeat.
+            udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+            # Set the socket to non-blocking so that it can time out.
+            udp_socket.setblocking(False)
+            # Wait thirty seconds for the UDP socket to timeout.
+            udp_socket.settimeout(5)
+            # Bind the socket. This will set a random  free port.
+            udp_socket.bind(("", 0))
+            start_commands.append({"$type": "start_udp",
+                                  "port": int(udp_socket.getsockname()[1])})
+
+        resp = self.communicate(start_commands)
+        self._is_standalone: bool = False
+        self._tdw_version: str = ""
+        self._unity_version: str = ""
+        self._done: bool = False
+        for r in resp[:-1]:
+            if Version.get_data_type_id(r) == "vers":
+                v = Version(r)
+                self._tdw_version = v.get_tdw_version()
+                self._unity_version = v.get_unity_version()
+                self._is_standalone = v.get_standalone()
+                break
+        # Start the UDP heartbeat in a thread.
+        if udp:
+            t = Thread(target=self._udp, args=([udp_socket]))
+            t.daemon = True
+            t.start()
+
         self.model_librarian: Optional[ModelLibrarian] = None
         self.scene_librarian: Optional[SceneLibrarian] = None
         self.material_librarian: Optional[MaterialLibrarian] = None
@@ -91,6 +132,16 @@ class Controller(object):
             self.socket.send_multipart(msg)
             resp = self.socket.recv_multipart()
 
+        # Check if we've received a quit signal. If we have, check if there was an error.
+        for i in range(len(resp) - 1):
+            r_id = OutputData.get_data_type_id(resp[i])
+            if r_id == "quit":
+                self._done = True
+                if not QuitSignal(resp[i]).get_ok():
+                    print("The build quit due to an error. Check the build log for more info.")
+                    self._print_build_log()
+                    break
+
         # Return the output data from the build.
         return resp
 
@@ -303,7 +354,7 @@ class Controller(object):
                 return v.get_tdw_version(), v.get_unity_version()
         if len(resp) == 1:
             raise Exception("Tried receiving version output data but didn't receive anything!")
-        raise Exception(f"Expected output data with ID vers but got: " + Version.get_data_type_id(resp[0]))
+        raise Exception(f"Expected output data with ID version but got: " + Version.get_data_type_id(resp[0]))
 
     @staticmethod
     def get_unique_id() -> int:
@@ -368,16 +419,60 @@ class Controller(object):
         Check the version of the build. If there is no build, download it.
         If the build is of the wrong version, recommend an upgrade.
 
-
         :param version: The version of TDW. You can set this to an arbitrary version for testing purposes.
         :param build_version: If not None, this overrides the expected build version. Only override for debugging.
         """
 
-        tdw_version, unity_version = self.get_version()
         # Override the build version for testing.
         if build_version is not None:
-            tdw_version = build_version
-        print(f"Build version {tdw_version}\nUnity Engine {unity_version}\nPython tdw module version {version}")
+            self._tdw_version = build_version
+        print(f"Build version {self._tdw_version}\nUnity Engine {self._unity_version}\n"
+              f"Python tdw module version {version}")
+
+    def _udp(self, s: socket.socket) -> None:
+        """
+        Do a UDP heartbeat. This is handled in a thread that is launched in the constructor.
+
+        :param s: The UDP listening socket.
+        """
+
+        # Get the ID of the controller process.
+        pid: int = os.getpid()
+        try:
+            while not self._done:
+                # If the main thread is down, stop.
+                if pid not in pids():
+                    self._done = True
+                    continue
+                sleep(0.1)
+                # Get the heartbeat.
+                try:
+                    s.recv(4)
+                # Timeout. Stop listening.
+                except socket.timeout:
+                    print("UDP heartbeat timeout. The build is probably down due to an unhandled exception."
+                          " Check the build log for more info.")
+                    self._print_build_log()
+                    self._done = True
+        finally:
+            # Close the socket.
+            s.close()
+            # Kill the controller process.
+            os.kill(pid, 9)
+
+    def _print_build_log(self) -> None:
+        """
+        Print a message indicating where the build log is located.
+        """
+
+        if self._is_standalone:
+            log_path = PLAYER_LOG_PATH
+        else:
+            log_path = EDITOR_LOG_PATH
+        print(f"If the build is on the same machine as this controller, the log path is probably "
+              f"{str(log_path.resolve())}")
+        print(f"If the build is on a remote Linux server, the log path is probably"
+              f" ~/.config/unity3d/MIT/TDW/Player.log (where ~ is your home directory)")
 
     @staticmethod
     def _check_pypi_version(v_installed_override: str = None, v_pypi_override: str = None) -> None:
