@@ -174,6 +174,17 @@ class HumanoidRecord(_Record):
         super().__init__(data)
 
 
+class RobotRecord(_Record):
+    """
+    A record for a robot asset bundle.
+    """
+
+    def __init__(self, data: Optional[dict] = None):
+        super().__init__(data)
+        self.source = data["source"]
+        self.immovable = data["immovable"]
+
+
 T = TypeVar("T", bound=_Record)
 
 
@@ -522,3 +533,16 @@ class HumanoidLibrarian(_Librarian[HumanoidRecord]):
 
     def _generate_record(self, data: dict) -> T:
         return HumanoidRecord(data)
+
+
+class RobotLibrarian(_Librarian[RobotRecord]):
+    """
+    Librarian class for robot metadata.
+    """
+
+    @staticmethod
+    def get_library_filenames() -> List[str]:
+        return ["robots.json"]
+
+    def _generate_record(self, data: dict) -> T:
+        return RobotRecord(data)
